# 2019/2/18 by DKZ

import json



def parseConf():
    with open('/etc/pccold.conf','r') as f:
        lines=f.readlines()
    j='{'
    for line in lines:
        l=line.replace('\n','').split('#')[0].strip()
        if l!='':
            k=l.split('=')[0].strip()
            v=l.split('=')[1].strip()
            j=j+'"'+k+'":'+v+','
    j=j[:-1]+'}'
    return json.loads(j)

class DottableDict(dict):
	def __init__(self,*args,**kwargs):
		dict.__init__(self,*args,**kwargs)
		self.__dict__=self
	def allowDotting(self,state=True):
		if state:
			self.__dict__=self
		else:
			self.__dict__=dict()

conf=DottableDict()
conf.allowDotting()
for k,v in parseConf().items():
	conf[k]=v


import re
import os
import logging
from .config import conf
import requests

room_obj_list=[]

from .tools import read,saveStream,ReturnCodeObserverThread,SleepKillerThread
from .bypyrm import doBypy

isinit=False

#log set
logging.basicConfig(level=logging.INFO,
                format='%(asctime)s [line:%(lineno)d] %(levelname)s %(message)s',
                datefmt='%m/%d %H:%M:%S',
                filename=conf.log_path,
                filemode='a')
# console = logging.StreamHandler()
# console.setLevel(logging.INFO)
# formatter = logging.Formatter('%(name)-12s: %(message)s')
# console.setFormatter(formatter)
# logging.getLogger('').addHandler(console)

def getRoomObjList():
    global room_obj_list
    global isinit
    global conf
    files=os.listdir(conf.download_path)
    if isinit:
        return room_obj_list
    logging.info('init room obj list')
    md=read(conf.videolist_path)
    lines=md.split('\n')
    for l in lines:
        match=re.match(r'\[(.*)\]\((.*)\)',l)
        if l and match:
            room_obj={'file_name':match.group(1)+'.mp4','url':match.group(2)}
            if room_obj.get('file_name','') in files:
                logging.info(room_obj.get('file_name','')+' is exist')
            else:
                room_obj_list.append(room_obj)
    isinit=True
    return room_obj_list
    

def downloadVideo():
    global conf
    print('videodownload main')
    room_obj_list=getRoomObjList()
    if len(room_obj_list)>0:
        room_obj=room_obj_list.pop()
        saveStream('source',room_obj.get('file_name','default.mp4'),url=room_obj.get('url',''))
    elif conf.is_bypy:
        doBypy()

def reqVideoList(author):
    global room_obj_list
    global isinit
    if isinit:
        return room_obj_list
    result=[]
    api='https://v.douyu.com/show/'
    url="https://v.douyu.com/video/author/getAuthorShowAndVideoList?up_id={author}".format(author=author)
    data=requests.get(url).json()
    ls=data.get('data').get('list')
    for l in ls:
        vls=l.get('video_list')
        for ll in vls:
            title=ll.get('title')
            print(title)
            file_name=re.sub(r"[\/\\\:\*\?\"\<\>\| \$\^\+\-\!]",'_',title)
            result.append({'url':api+ll.get('hash_id'),'file_name':file_name+'.mp4'})
    room_obj_list=result
    isinit=True
    return result

def download3DaysVideo():
    global conf
    room_obj_list=reqVideoList(conf.video_author)
    if len(room_obj_list)>0:
        room_obj=room_obj_list.pop()
        saveStream('source',room_obj.get('file_name','default.mp4'),url=room_obj.get('url',''))
    elif conf.is_bypy:
        doBypy()

if __name__ == '__main__':
    downloadVideo()



# coding: utf-8
import smtplib
from email.mime.text import MIMEText
import time
import requests
import json
import subprocess
import threading
import os
import signal
import logging
import traceback
from .config import conf

pidpool={}

def sendEmails(room_obj):
    global conf
    logging.info('sendEmails')
    try:
        email_obj=initPcColdEmail(room_obj)
        sendEmail(email_obj['subj'],conf.my_email,email_obj['body'],conf.mail_sender,conf.mail_passwd,conf.mail_host,conf.mail_port)
    except Exception as e:
        logging.warning('*** fail send email')
        logging.warning(e)
        tb=traceback.format_exc()
        logging.warning(tb)

def saveStream(level,file_name,url):
    global conf
    url=url or "https://www.douyu.com/"+str(conf.room_id)
    logging.info('saveStream')
    cmd='pccoldcli '+conf.download_path+'/'+file_name
    shell=subprocess.Popen(cmd,shell=True,preexec_fn=os.setsid)
    logging.info('save start pid='+str(shell.pid))
    logging.info('$ '+cmd)
    sleepkiller=SleepKillerThread(shell)
    returncode_observer=ReturnCodeObserverThread(shell)
    returncode_observer.sleepkiller=sleepkiller
    sleepkiller.return_code_observer=returncode_observer
    global pidpool
    pidpool[str(shell.pid)]=True


class ReturnCodeObserverThread():
    isstoped=False
    shell=None
    thread=None
    sleepkiller=None

    def __init__(self,shell):
        self.shell=shell
        self.thread=threading.Thread(target=self.returnCodeObserver)
        self.thread.start()

    def stop(self):
        self.isstoped=True

    def returnCodeObserver(self):
        logging.info('returnCodeObserver')
        returncode=self.shell.wait()
        global pidpool
        del pidpool[str(self.shell.pid)]
        self.sleepkiller.stop()
        logging.info('save quit pid='+str(self.shell.pid)+' return code='+str(returncode))
        if returncode!=-9 and not self.isstoped:
            time.sleep(60)
            logging.info('start main from return code observer')
            ReturnCodeObserverThread.main()

class SleepKillerThread():
    isstoped=False
    shell=None
    thread=None
    return_code_observer=None

    def __init__(self,shell):
        self.shell=shell
        self.thread=threading.Thread(target=self.sleepKiller)
        self.thread.start()

    def stop(self):
        self.isstoped=True

    def sleepKiller(self):
        global conf
        if not conf.is_cut:
            return
        logging.info('sleepKiller')
        time.sleep(conf.how_long)
        if self.isstoped:
            return
        t=threading.Thread(target=SleepKillerThread.main)
        t.start()
        self.return_code_observer.stop()
        time.sleep(60)
        try:
            os.killpg(os.getpgid(self.shell.pid),signal.SIGKILL)
            logging.info('save end '+str(self.shell.pid))
        except Exception as e:
            logging.info('*** save end err '+str(self.shell.pid))


def testRoomStatus():
    global conf
    try:
        room_obj=requests.get("https://www.douyu.com/betard/"+str(conf.room_id),timeout=10).json()
        result=room_obj.get('room',{'show_status':0})
        return result
    except Exception as e:
        logging.info('*** test room status err')
        return {'show_status':0}

def initPcColdEmail(roomobj):
    global conf
    subj='[pccold]'+roomobj.get('room_name')+'@'+roomobj.get('nickname')
    body='\nenv:'+conf.env
    body+='\nroom_name:'+roomobj.get('room_name')
    body+='\nowner_name:'+roomobj.get('owner_name')+'#'+str(roomobj.get('room_id'))
    t=time.localtime(roomobj.get('show_time'))
    start_time=time.strftime("%Y-%m-%d %H:%M:%S", t)
    body+='\nstart_time:'+start_time
    body+='\nsecond_lvl_name:'+roomobj.get("second_lvl_name")
    body+='\nlink:http://www.douyutv.com/'+str(roomobj.get('room_id'))
    body+=conf.pccold_contact
    return {'body':body,'subj':subj}

def sendEmail(subj,to,body,sender,passwd,host,port=25):
    msg=MIMEText(body,'plain','utf-8')
    msg['subject']=subj
    msg['from']=sender
    msg['to']=to

    server=smtplib.SMTP(host,port)
    server.login(sender,passwd)
    server.sendmail(sender,to,msg.as_string())
    server.quit()

def read(file):
    with open(file,'r') as f:
        return f.read()

def write(file,str):
    with open(file,'w') as f:
        f.write(str)

if __name__ == '__main__':
    testRoomStatus()


import websocket
import socket
import time
from websocket import create_connection,WebSocket
import pystt
import threading
from .config import conf
from .tail_call import tail_call_optimized
from .tools import sendEmail

luckystar=0
keybody=''
uenter_set=set()
speaker_set=set()





class DouyuMsg(object):
    """
    解析与封装信息格式见斗鱼开放平台
    https://open.douyu.com/source/api/63"""
    def __init__(self,obj=None, message=None):
        self.obj = obj
        self.message = message

    def objToMessage(self):
        content = pystt.dumps(self.obj)
        content_byte = bytes(content.encode('utf-8'))
        content_length = len(content_byte) + 8 + 1
        length_byte = int.to_bytes(content_length, length=4, byteorder='little')
	  
        magic = bytearray([0xb1, 0x02])
        zero_byte = bytearray([0x00])
        return length_byte + length_byte + magic + zero_byte + zero_byte + content_byte + zero_byte 

    def messageToInfos(self):
        pos = 0
        infos = [ ]
        while pos < len(self.message):
            content_length = int.from_bytes(self.message[pos: pos + 4], byteorder='little')
            content = self.message[pos + 12: pos + 4 + content_length - 1].decode(encoding='utf-8', errors='ignore')
            self.obj = pystt.loads(content)
            infos.append(self.getInfo())
            pos += (4 + content_length)
        # print ("Receive {} messages".format(len(infos)))
        return infos

    def getInfo(self):
        if self.obj.get('type')=='chatmsg':
            self.chatMsgFillter()
            return self.obj.get('uid','')+' @ '+self.obj.get('nn','')+' : '+self.obj.get('txt','')
        elif self.obj.get('type')=='uenter':
            self.enterFillter()
            return self.obj.get('uid','')+' @ '+self.obj.get('nn','')+' @ uenter'
        elif self.obj.get('type')=='rss':
            self.onLive()
            return '### Live ###'
        # elif self.obj.get('type') in 'loginres':
        #     return self.obj
        elif self.obj.get('type') in 'loginres,dgb,wiru,rankup,actfsts1od_r,frank,rri,svsnres,newblackres,fire_user,fire_start,tsboxb,ghz2019arkcalc,ghz2019s1info,ghz2019s2info,fire_real_user,gbroadcast,srres,spbc,ghz2019s2calc,upgrade,rquizisn,anbc,wirt,ghz2019s1disp,blab,cthn,rnewbc,noble_num_info,rank_change,mrkl,synexp,fswrank,ranklist,qausrespond':
            return None
        else:
            print('*** '+self.obj.get('type')+' ***')
            return None

    def onLive(self):
        global keybody
        global uenter_set
        global speaker_set
        keybody='\nenter:'+str(len(uenter_set))+'\nspeaker:'+str(len(speaker_set))+'\n\n'+keybody+'\n\n'
        # t=threading.Thread(target=sendEmail,args=('[pccold]Live',conf.my_email,conf.env+'\n\n\n'+keybody+'\n\n\n'+conf.pccold_contact,conf.mail_sender,conf.mail_passwd,conf.mail_host,conf.mail_port,))
        # t.start()
        print(speaker_set)
        keybody=''
        uenter_set=set()
        speaker_set=set()

    def enterFillter(self):
        global uenter_set
        uenter_set.add(self.obj.get('nn',''))
        if self.obj.get('uid','')==conf.keyuid or self.obj.get('nn','')==conf.keyuser:
            print('### PcCold Enter ###')
            t=threading.Thread(target=sendEmail,args=('[pccold]'+conf.keyuser+' Enter',conf.my_email,conf.env+'\n'+conf.pccold_contact,conf.mail_sender,conf.mail_passwd,conf.mail_host,conf.mail_port,))
            t.start()

    def chatMsgFillter(self):
        global keybody
        global speaker_set
        txt=self.obj.get('txt','')
        uname=self.obj.get('nn','')
        uid=self.obj.get('uid','')
        speaker_set.add(uname)
        keywordlist=conf.keyword.split(',')
        for w in keywordlist:
            if w in txt:
                keybody+=uid+' @ '+uname+' : '+txt+'\n'
        if uname==conf.keyuser or uid==conf.keyuid:
            keybody+=uid+' @ '+uname+' : '+txt+'\n'
        # ⭐ 🌟
        global luckystar
        if '⭐' in txt or '🌟' in txt:
            if luckystar<5 and luckystar>=0:
                luckystar+=1
            elif luckystar>=5: 
                luckystar=-1
                print('### luckystar ###')
                t=threading.Thread(target=sendEmail,args=('[pccold]Lucky Star',conf.my_email,conf.env+'\n'+conf.pccold_contact,conf.mail_sender,conf.mail_passwd,conf.mail_host,conf.mail_port,))
                t.start()


def login(ws,room_id,username,uid):
    req={
        'type':'loginreq',
        'room_id':room_id,
        'dfl':'sn@A=105@Sss@A=1',
        'username':username,
        'uid':uid,
        'ver':'20190610',
        'aver':'218101901',
        'ct':'0'
    }
    # print('### login ###',req)
    binary = DouyuMsg(req).objToMessage()
    ws.send(binary)

def join(ws,room_id):
    req={
        'type':'joingroup',
        'rid':room_id,
        'gid':'1'
    }
    # print('### join ###',req)
    binary = DouyuMsg(req).objToMessage()
    ws.send(binary)

@tail_call_optimized
def mrkl(ws):
    global luckystar
    luckystar=0
    req={
        'type':'mrkl'
    }
    binary = DouyuMsg(req).objToMessage()
    now_time=time.strftime('## %m_%d_%H_%M ##',time.localtime(time.time()))
    print(now_time)
    try:
        ws.send(binary)
        time.sleep(45)
    except Exception as err:
        print('** mrkl error **')
        return None
    return mrkl(ws)


def keepalive(ws):
    t=threading.Thread(target=mrkl,args=(ws,))
    t.start()

def on_message(ws, message):
    try:
        for info in DouyuMsg(message=message).messageToInfos():
            if info:
                print(info)
    except Exception as err:
        print('** message parse err **')
        print(message,err)

def on_error(ws, error):
    print(error)

def on_close(ws):
    print("### closed ###")
    t=threading.Thread(target=wsdanmumain)
    t.start()

def on_open(ws):
    print('### open ###')
    login(ws,str(conf.room_id),conf.username,conf.uid)
    join(ws,str(conf.room_id))
    keepalive(ws)

def wsdanmumain():
    # websocket.enableTrace(True)
    ws = websocket.WebSocketApp("wss://danmuproxy.douyu.com:8503/",
                              on_message = on_message,
                              on_error = on_error,
                              on_close = on_close)
    ws.on_open = on_open
    ws.run_forever()

if __name__ == '__main__':
    wsdanmumain()



from .pccold import main
from .videodownload import downloadVideo,download3DaysVideo
from .tools import ReturnCodeObserverThread,SleepKillerThread
from .wsdanmu import wsdanmumain
from .pccold_cli import cli_main




import subprocess
import hashlib
import re
import time
import uuid

from requests.adapters import HTTPAdapter
from streamlink.plugin import Plugin
from streamlink.plugin.api import http, validate, useragents
from streamlink.stream import HTTPStream, HLSStream, RTMPStream

import requests
import execjs
from pccold.config import conf

# https://github.com/wbt5/real-url/blob/master/douyu.py

def get_tt():
    tt1 = str(int(time.time()))
    tt2 = str(int((time.time() * 1000)))
    today = time.strftime('%Y%m%d', time.localtime())
    return tt1, tt2, today


def get_homejs(rid):
    room_url = 'https://m.douyu.com/' + rid
    response = requests.get(url=room_url)
    pattern_real_rid = r'"rid":(\d{1,7})'
    real_rid = re.findall(pattern_real_rid, response.text, re.I)[0]
    if real_rid != rid:
        room_url = 'https://m.douyu.com/' + real_rid
        response = requests.get(url=room_url)
    homejs = ''
    pattern = r'(function ub9.*)[\s\S](var.*)'
    result = re.findall(pattern, response.text, re.I)
    str1 = re.sub(r'eval.*;}', 'strc;}', result[0][0])
    homejs = str1 + result[0][1]
    return homejs, real_rid


def get_sign(rid, post_v, tt, ub9):
    docjs = execjs.compile(ub9)
    res2 = docjs.call('ub98484234')
    str3 = re.sub(r'\(function[\s\S]*toString\(\)', '\'', res2)
    md5rb = hashlib.md5((rid + '10000000000000000000000000001501' + tt + '2501' +
                         post_v).encode('utf-8')).hexdigest()
    str4 = 'function get_sign(){var rb=\'' + md5rb + str3
    str5 = re.sub(r'return rt;}[\s\S]*','return re;};', str4) 
    str6 = re.sub(r'"v=.*&sign="\+', '', str5)
    docjs1 = execjs.compile(str6)
    sign = docjs1.call(
        'get_sign', rid, '10000000000000000000000000001501', tt)
    return sign


def mix_room(rid):
    result1 = 'PKing'
    return result1


def get_pre_url(rid, tt):
    request_url = 'https://playweb.douyucdn.cn/lapi/live/hlsH5Preview/' + rid
    post_data = {
        'rid': rid,
        'did': '10000000000000000000000000001501'
    }
    auth = hashlib.md5((rid + str(tt)).encode('utf-8')).hexdigest()
    header = {
        'content-type': 'application/x-www-form-urlencoded',
        'rid': rid,
        'time': tt,
        'auth': auth
    }
    response = requests.post(url=request_url, headers=header, data=post_data)
    response = response.json()
    pre_url = ''
    if response.get('error') == 0:
        real_url = (response.get('data')).get('rtmp_live')
        if 'mix=1' in real_url:
            pre_url = mix_room(rid)
        else:
            pattern1 = r'^[0-9a-zA-Z]*'
            pre_url = re.search(pattern1, real_url, re.I).group()
    return pre_url


def get_sign_url(post_v, rid, tt, ub9):
    sign = get_sign(rid, post_v, tt, ub9)
    request_url = 'https://m.douyu.com/api/room/ratestream'
    post_data = {
        'v': '2501' + post_v,
        'did': '10000000000000000000000000001501',
        'tt': tt,
        'sign': sign,
        'ver': '219032101',
        'rid': rid,
        'rate': '1'
    }
    header = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'User-Agent': 'Mozilla/5.0 (Linux; Android 5.0; SM-G900P Build/LRX21T) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Mobile Safari/537.36'
    }
    response = requests.post(url=request_url, headers=header, data=post_data).json()
    print('resp',response)
    if response.get('code') == 0:
        real_url = (response.get('data')).get('url')
        if 'mix=1' in real_url:
            result1 = mix_room(rid)
        else:
            pattern1 = r'live/(\d{1,8}[0-9a-zA-Z]+)_?[\d]{0,4}/playlist'
            result1 = re.findall(pattern1, real_url, re.I)[0]
    else:
        result1 = 0
    return result1


def get_real_url(rid):
    rid = str(rid)
    tt = get_tt()
    url = get_pre_url(rid, tt[1])
    if url:
        return "http://tx2play1.douyucdn.cn/live/" + url +"_"+conf.stream_type +".flv?uuid="
    else:
        result = get_homejs(rid)
        real_rid = result[1]
        homejs = result[0]
        real_url = get_sign_url(tt[2], real_rid, tt[0], homejs)
        if real_url != 0:
            real_url = "http://tx2play1.douyucdn.cn/live/" + real_url + "_"+conf.stream_type +".flv?uuid="
        else:
            real_url = '未开播'
        return real_url

def get_url_from_js(rid):
    # 从播放页源代码中直接匹配地址
    header = {
    'Content-Type': 'application/x-www-form-urlencoded'
    }
    try:
        response = requests.get('https://www.douyu.com/{}'.format(rid), headers=header).text
        real_url = re.findall(r'live/({}[\d\w]*?)_'.format(rid), response)[0]
    except:
        real_url = '直播间未开播或不存在'
    return "http://tx2play1.douyucdn.cn/live/" + real_url +"_"+conf.stream_type +".flv?uuid="

# puppteer

def getPlay():
    js=subprocess.Popen(['node','index.js'],stdout=subprocess.PIPE)
    stdout,stderr=js.communicate()
    returncode=js.wait() # 0 ok , 1 not get url , 3 time out , 4 error
    print('return code',returncode)
    print(stdout.decode())
    if returncode==0 and '$$$$' in stdout.decode():
        return stdout.decode().split('$$$$')[1]
    else:
        exit(returncode)


# plugin

# rtmp_url=getPlay()
# rtmp_url=get_real_url(9999) #room id

STREAM_WEIGHTS = {
    'default':1
}

_url_re = re.compile(r"""
    http(s)?://
    (?:
        (?P<subdomain>.+)
        \.
    )?
    douyu.com/
    (?:
        show/(?P<vid>[^/&?]+)|
        (?P<channel>[^/&?]+)
    )
""", re.VERBOSE)


class PcCold(Plugin):
    @classmethod
    def can_handle_url(cls, url):
        return url=='https://pccold'
        # return _url_re.match(url)

    @classmethod
    def stream_weight(cls, stream):
        if stream in STREAM_WEIGHTS:
            return STREAM_WEIGHTS[stream], "pccold"
        return Plugin.stream_weight(stream)

    def _get_streams(self):  
        rtmp_url=get_real_url(conf.room_id) #room id
        if 'rtmp:' in rtmp_url:
            stream = RTMPStream(self.session, {
                    "rtmp": rtmp_url,
                    "live": True
                    })
            yield 'default', stream
        else:
            yield 'default', HTTPStream(self.session, rtmp_url)

__plugin__ = PcCold 


import sys
import time
 
class TailRecurseException(Exception):
    def __init__(self, args, kwargs):
        self.args = args
        self.kwargs = kwargs
 
def tail_call_optimized(g):
    def func(*args, **kwargs):
        f = sys._getframe()
        if f.f_back and f.f_back.f_back \
            and f.f_back.f_back.f_code == f.f_code:
            raise TailRecurseException(args, kwargs)
        else:
            while 1:
                try:
                    return g(*args, **kwargs)
                except TailRecurseException as e:
                    args = e.args
                    kwargs = e.kwargs
    return func


import streamlink_cli.main as cli
import streamlink
import sys
import os
from .config import conf


def cli_main(output):

    error_code = 0
    parser = cli.build_parser()

    cli.setup_args(parser, ignore_unknown=True)
    cli.args.output=output
    cli.args.url='https://pccold'
    cli.args.stream=['default']
    # print(cli.args)


    # Console output should be on stderr if we are outputting
    # a stream to stdout.
    if cli.args.stdout or cli.args.output == "-":
        console_out = sys.stderr
    else:
        console_out = sys.stdout

    # We don't want log output when we are printing JSON or a command-line.
    silent_log = any(getattr(cli.args, attr) for attr in cli.QUIET_OPTIONS)
    log_level = cli.args.loglevel if not silent_log else "none"
    cli.setup_logging(console_out, log_level)
    cli.setup_console(console_out)

    cli.setup_streamlink()
    cli.setup_plugin_args(cli.streamlink, parser)
    path=os.path.abspath(os.path.dirname(__file__))[:-6]+'plugins'
    print('dir',path)
    cli.load_plugins([path])

    # update the logging level if changed by a plugin specific config
    log_level = cli.args.loglevel if not silent_log else "none"
    cli.logger.root.setLevel(log_level)

    cli.setup_http_session()

    try:
        cli.setup_options()
        cli.handle_url()
    except KeyboardInterrupt:
        # Close output
        if cli.output:
            cli.output.close()
        cli.console.msg("Interrupted! Exiting...")
        error_code = 130
    finally:
        if cli.stream_fd:
            try:
                cli.log.info("Closing currently open stream...")
                cli.stream_fd.close()
            except KeyboardInterrupt:
                error_code = 130
   
    sys.exit(error_code)


# 2018/12/31 by DKZ

from .tools import sendEmail
import psutil
import subprocess
import logging
from .config import conf


def doBypy():
    global conf
    logging.info('doBypy')
    cmd='cd '+conf.download_path+';bypy upload'
    logging.info('$ '+cmd)
    shell=subprocess.Popen(cmd,shell=True)
    return shell

def psCheck(name):
    result=[p.info for p in psutil.process_iter(attrs=['pid', 'name']) if name in p.info['name']]
    return result

def initBypyRmEmail(body):
    global conf
    subj='[pccold] bypyrm '+body
    logging.info('[bypyrm] '+body)
    try:
        sendEmail(subj,conf.my_email,body+'\n'+conf.env+'\n'+conf.pccold_contact,conf.mail_sender,conf.mail_passwd,conf.mail_host,conf.mail_port)
    except Exception as e:
        logging.info('*** email fail',e)

def bypyrm():
    global conf
    logging.info('start bypyrm')
    if not psCheck('bypy') and not psCheck('streamlink'):
        shell=doBypy()
        returncode=shell.wait()
        if returncode==0:
            cmd='cd '+conf.download_path+';rm *.mp4'
            p=subprocess.Popen(cmd,shell=True)
            if conf.bypy_mail:
                initBypyRmEmail('ok')
        else:
            initBypyRmEmail('fail '+str(returncode))
    else:
        initBypyRmEmail('psCheck process is runing')


if __name__ == '__main__':
    bypyrm()     


# 2019/2/13 by DKZ

from setuptools import setup

setup(name='pccold',
    version='0.31',
    packages=['pccold','plugins'],
    author='davidkingzyb',
    author_email='davidkingzyb@qq.com',
    license='MIT',
    url='https://pypi.python.org/pypi',
    data_files = [('/etc', ['etc/pccold.conf'])],
    keywords="douyu pccold",
    install_requires=[
        'streamlink==0.14.2',
        'bypy>=1.6.4',
        'psutil>=5.5.0',
        'pystt==0.2.2',
        'websocket-client>=0.54.0',
        'PyExecJS>=1.5.1'
    ],
    scripts=['./bin/pccold',
    './bin/pccoldvideo',
    './bin/pccoldvideolist',
    './bin/pccolddanmu',
    './bin/pccoldcli']
    )

# $ python3 setup.py sdist
# $ python3 setup.py sdist bdist_wheel
# $ twine upload dist/pccold-0.1-py3-none-any.whl 


# coding: utf-8
"""
===================================================
                                                   
                                     __            
 _______  ______   ______    _____  |  |       ___ 
|   __  ||   ___| |   ___|  /     \ |  |   ___|   |
|    ___||  |____ |  |____ |   o   ||  |_ |  ___  |
|___|    |_______||_______| \_____/ |____||_______|
===================================================
2018/10/5 by DKZ https://davidkingzyb.tech

"""

import json
import time
import subprocess
import threading
import logging
import sys
import os
import signal
import traceback
import re
import requests

from .tools import sendEmails,saveStream,testRoomStatus,pidpool,ReturnCodeObserverThread,SleepKillerThread
from .bypyrm import psCheck,initBypyRmEmail,bypyrm,doBypy
from .config import conf

is_live=False

#log set
logging.basicConfig(level=logging.INFO,
                format='%(asctime)s [line:%(lineno)d] %(levelname)s %(message)s',
                datefmt='%m/%d %H:%M:%S',
                filename=conf.log_path,
                filemode='a')
# console = logging.StreamHandler()
# console.setLevel(logging.INFO)
# formatter = logging.Formatter('%(name)-12s: %(message)s')
# console.setFormatter(formatter)
# logging.getLogger('').addHandler(console)


def main(): 
    global conf
    try:
        global is_live
        room_obj=testRoomStatus()
        logging.info('show_status:'+str(room_obj.get('show_status'))+' videoLoop:'+str(room_obj.get('videoLoop')))
        if room_obj.get('show_status')==1 and room_obj.get('videoLoop')==0:
            logging.info('live on')
            if not is_live:
                is_live=True
                t=threading.Thread(target=sendEmails,args=(room_obj,))
                t.start()
            now_time=time.strftime('_%m_%d_%H_%M',time.localtime(time.time()))
            room_name=room_obj.get('room_name','default')
            room_name=re.sub(r"[\/\\\:\*\?\"\<\>\| \$\^\+\-\!]",'_',room_name)
            saveStream(conf.stream_type,room_name+now_time+'.mp4',None)
        elif room_obj.get('show_status')==2:
            time.sleep(90)  
            tt=threading.Thread(target=main)
            tt.start()
            if is_live:
                is_live=False
                if conf.is_bypy:
                    shell=doBypy()
                    returncode=shell.wait()
                    logging.info('bypy returncode '+str(returncode))
                    if returncode==0 and conf.is_bypy_rm:
                        bypyrm()
        else:
            time.sleep(90)
            tt=threading.Thread(target=main)
            tt.start()
    except Exception as e:
        logging.warning('*** main fail')
        logging.warning(e)
        tb=traceback.format_exc()
        logging.warning(tb)
        pp=pidpool.copy()
        for k,v in pp.items():
            try:
                os.killpg(os.getpgid(int(k)),signal.SIGKILL)
                logging.info('main kill '+k)
            except Exception as e:
                logging.info('*** main kill err '+k)
        time.sleep(60)
        ttt=threading.Thread(target=main)
        ttt.start()

ReturnCodeObserverThread.main=main
SleepKillerThread.main=main

if __name__ == '__main__':
    logging.info('start pccold')
    main()
